#!/usr/bin/env python

from datetime import datetime
from imiptools.config import STORE_DIR, PUBLISH_DIR
from os.path import abspath, commonprefix, exists, join, split
from os import makedirs
from vCalendar import iterwrite

def check_dir(base, dir):
    return commonprefix([base, abspath(dir)]) == base

def make_calendar(fragment, method=None):
    return ("VCALENDAR", {},
            (method and [("METHOD", {}, method)] or []) +
            [("VERSION", {}, "2.0")] +
            fragment
           )

def to_stream(out, fragment, encoding="utf-8"):
    iterwrite(out, encoding=encoding).append(fragment)

class FileBase:

    "Basic filesystem operations."

    def __init__(self, store_dir=STORE_DIR):
        self.store_dir = store_dir
        if not exists(self.store_dir):
            makedirs(self.store_dir)

    def get_file_object(self, base, *parts):
        pathname = join(base, *parts)
        return check_dir(base, pathname) and pathname or None

    def get_object_in_store(self, *parts):

        """
        Return the name of any valid object stored within a hierarchy specified
        by the given 'parts'.
        """

        parent = expected = self.store_dir

        for part in parts:
            filename = self.get_file_object(expected, part)
            if not filename:
                return False
            parent = expected
            expected = filename

        if not exists(parent):
            makedirs(parent)

        return filename

class FileStore(FileBase):

    "A file store of tabular free/busy data and objects."

    def get_event(self, user, uid):

        "Get the event for the given 'user' with the given 'uid'."

        filename = self.get_object_in_store(user, uid)
        if not filename or not exists(filename):
            return None

        return exists(filename) and open(filename) or None

    def set_event(self, user, uid, node):

        "Set an event for 'user' having the given 'uid' and 'node'."

        filename = self.get_object_in_store(user, uid)
        if not filename:
            return False

        f = open(filename, "w")
        try:
            to_stream(f, node)
        finally:
            f.close()

        return True

    def get_freebusy(self, user):

        "Get free/busy details for the given 'user'."

        filename = self.get_object_in_store(user, "freebusy")
        if not filename or not exists(filename):
            return None

        f = open(filename)
        try:
            l = []
            for line in f.readlines():
                l.append(tuple(line.strip().split("\t")))
            return l
        finally:
            f.close()

    def set_freebusy(self, user, freebusy):

        "For the given 'user', set 'freebusy' details."

        filename = self.get_object_in_store(user, "freebusy")
        if not filename:
            return False

        f = open(filename, "w")
        try:
            for item in freebusy:
                f.write("\t".join(item) + "\n")
        finally:
            f.close()

        return True

    def get_requests(self, user):

        "Get requests for the given 'user'."

        filename = self.get_object_in_store(user, "requests")
        if not filename:
            return None

        f = open(filename)
        try:
            return [line.strip() for line in f.readlines()]
        finally:
            f.close()

    def set_requests(self, user, requests):

        "For the given 'user', set the list of queued 'requests'."

        filename = self.get_object_in_store(user, "requests")
        if not filename:
            return False

        f = open(filename, "w")
        try:
            for request in requests:
                print >>f, request
        finally:
            f.close()

        return True

    def set_request(self, user, request):

        "For the given 'user', set the queued 'request'."

        filename = self.get_object_in_store(user, "requests")
        if not filename:
            return False

        f = open(filename, "a")
        try:
            print >>f, request
        finally:
            f.close()

        return True

    def queue_request(self, user, uid):

        "Queue a request for 'user' having the given 'uid'."

        requests = self.get_requests(user)

        if uid not in requests:
            return self.set_request(user, uid)

        return False

class FilePublisher(FileBase):

    "A publisher of objects."

    def __init__(self, store_dir=PUBLISH_DIR):
        FileBase.__init__(self, store_dir)

    def set_freebusy(self, user, freebusy):

        "For the given 'user', set 'freebusy' details."

        filename = self.get_object_in_store(user, "freebusy")
        if not filename:
            return False

        record = []
        rwrite = record.append

        rwrite(("ORGANIZER", {}, user))
        rwrite(("UID", {}, user))
        rwrite(("DTSTAMP", {}, datetime.utcnow().strftime("%Y%m%dT%H%M%SZ")))

        for start, end, uid in freebusy:
            rwrite(("FREEBUSY", {"FBTYPE" : "BUSY"}, "/".join([start, end])))

        f = open(filename, "w")
        try:
            to_stream(f, make_calendar([("VFREEBUSY", {}, record)], "PUBLISH"))
        finally:
            f.close()

        return True

# vim: tabstop=4 expandtab shiftwidth=4
