#!/usr/bin/env python

from email import message_from_file
from email.mime.message import MIMEMessage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from smtplib import LMTP, SMTP
from imiptools.content import handle_itip_part
import sys

MESSAGE_SENDER = "resources+agent@example.com"

MESSAGE_SUBJECT = "Calendar system message"

MESSAGE_TEXT = """\
This is a response to a calendar message sent by your calendar program.
"""

# Postfix exit codes.

EX_TEMPFAIL     = 75

# Permitted iTIP content types.

itip_content_types = [
    "text/calendar",                        # from RFC 6047
    "text/x-vcalendar", "application/ics",  # other possibilities
    ]

# Sending of outgoing messages.

def sendmail(sender, recipients, data, lmtp_socket=None):
    if lmtp_socket:
        smtp = LMTP(lmtp_socket)
    else:
        smtp = SMTP("localhost")
    smtp.sendmail(sender, recipients, data)
    smtp.quit()

def make_message(parts, recipients, sender, subject, body_text):
    message = MIMEMultipart("mixed", _subparts=parts)
    message.preamble = body_text
    payload = message.get_payload()
    payload.insert(0, MIMEText(body_text))

    message["From"] = sender
    for recipient in recipients:
        message["To"] = recipient
    message["Subject"] = subject

    return message

# Processing of incoming messages.

def get_all_values(msg, key):
    l = []
    for v in msg.get_all(key) or []:
        l += [s.strip() for s in v.split(",")]
    return l

class Processor:

    "The processing framework."

    def __init__(self, handlers, sender=None, subject=None, body_text=None):
        self.handlers = handlers
        self.sender = sender or MESSAGE_SENDER
        self.subject = subject or MESSAGE_SUBJECT
        self.body_text = body_text or MESSAGE_TEXT
        self.lmtp_socket = None

    def process(self, f, original_recipients, recipients):

        """
        Process content from the stream 'f' accompanied by the given
        'original_recipients' and 'recipients'.
        """

        msg = message_from_file(f)
        senders = msg.get_all("Reply-To") or msg.get_all("From")
        original_recipients = original_recipients or get_all_values(msg, "To")

        # Handle messages with iTIP parts.

        all_responses = []
        handled = False

        for part in msg.walk():
            if part.get_content_type() in itip_content_types and \
                part.get_param("method"):

                all_responses += handle_itip_part(part, original_recipients, self.handlers)
                handled = True

        # Pack any returned parts into a single message.

        if all_responses:
            outgoing_parts = []
            forwarded_parts = []

            for outgoing, part in all_responses:
                if outgoing:
                    outgoing_parts.append(part)
                else:
                    forwarded_parts.append(part)

            # Reply using any outgoing parts in a new message.

            if outgoing_parts:
                message = self.make_message(outgoing_parts, senders)

                if "-d" in sys.argv:
                    print message
                else:
                    sendmail(self.sender, senders, message.as_string())

            # Forward messages to their recipients using the existing message.

            if forwarded_parts:
                message = self.wrap_message(msg, forwarded_parts)

                if "-d" in sys.argv:
                    print message
                elif self.lmtp_socket:
                    sendmail(self.sender, original_recipients, message.as_string(), self.lmtp_socket)

        # Unhandled messages are delivered as they are.

        if not handled:
            if "-d" in sys.argv:
                print msg
            elif self.lmtp_socket:
                sendmail(self.sender, original_recipients, msg.as_string(), self.lmtp_socket)

    def make_message(self, parts, recipients):

        "Make a message from the given 'parts' for the given 'recipients'."

        return make_message(parts, recipients, self.sender, self.subject, self.body_text)

    def wrap_message(self, msg, parts):

        "Wrap 'msg' and provide the given 'parts' as the primary content."

        message = MIMEMultipart("mixed", _subparts=parts)
        message.preamble = self.body_text
        message.get_payload().append(MIMEMessage(msg))

        message["From"] = msg["From"]
        message["To"] = msg["To"]
        message["Subject"] = msg["Subject"]

        return message

    def process_args(self, args, stream):

        """
        Interpret the given program arguments 'args' and process input from the
        given 'stream'.
        """

        # Obtain the different kinds of recipients plus sender address.

        original_recipients = []
        recipients = []
        senders = []
        lmtp = []

        l = []

        for arg in args:

            # Switch to collecting recipients.

            if arg == "-o":
                l = original_recipients
            elif arg == "-r":
                l = recipients

            # Switch to collecting senders.

            elif arg == "-s":
                l = senders

            # Switch to getting the LMTP socket.

            elif arg == "-l":
                l = lmtp

            # Ignore debugging options.

            elif arg == "-d":
                pass
            else:
                l.append(arg)

        self.sender = senders and senders[0] or self.sender
        self.lmtp_socket = lmtp and lmtp[0] or None
        self.process(stream, original_recipients, recipients)

    def __call__(self):

        """
        Obtain arguments from the command line to initialise the processor
        before invoking it.
        """

        args = sys.argv[1:]

        if "-d" in args:
            self.process_args(args, sys.stdin)
        else:
            try:
                self.process_args(args, sys.stdin)
            except SystemExit, value:
                sys.exit(value)
            except Exception, exc:
                if "-v" in args:
                    raise
                type, value, tb = sys.exc_info()
                print >>sys.stderr, "Exception %s at %d" % (exc, tb.tb_lineno)
                sys.exit(EX_TEMPFAIL)
        sys.exit(0)

# vim: tabstop=4 expandtab shiftwidth=4
